--         FILE NAME: bootrom.vhdl
--       ENTITY NAME: boot_rom
-- ARCHITECTURE NAME: behave
--          REVISION: A
--
--       DESCRIPTION: 64 byte x 8 bit ROM to down a Monitor
--                    program on reset
--
--Written by John Kent for the micro8 processor

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
-- use work.memory.all;

entity boot_rom is
  port (
    WB_ADR_I : in  std_logic_vector(5 downto 0);
    WB_DAT_O : out std_logic_vector(7 downto 0);
	 WB_STB_I : in  std_logic;
	 WB_ACK_O : out std_logic
  );
end entity boot_rom;

architecture basic of boot_rom is
  constant width   : integer := 8;
  constant memsize : integer := 64;

  type rom_array is array(0 to memsize-1) of std_logic_vector(width-1 downto 0);

  constant rom_data : rom_array :=
  ( "10100000", "11111111",             -- 0000 - a0 ff    RESET NORX #$FF
    "00100000", "11111111",             -- 0002 - 20 ff    POLL1 NORA #$FF
    "00001111", "11111101",             -- 0004 - 0f fd          ADDA UARTCR
    "00100000", "11111110",             -- 0006 - 20 fe          NORA #not(TXBE)
	 "11101111", "11111000",             -- 0008 - ef f8          BNE POLL1
	 "00100000", "11111111",             -- 000a - 20 ff          NORA #$FF
    "00010000", "00110000",             -- 000c - 10 30          ADDA MSG,X
	 "01101000", "00000110",             -- 000e - 68 06          BEQ POLL2
	 "10000000", "00000001",             -- 0010 - 80 01          ADDX #1
	 "01001111", "11111100",             -- 0012 - 4f fc          STA UARTDR
    "11101111", "11101100",             -- 0014 - ef ec          BNE POLL1
	 "00100000", "11111111",             -- 0016 - 20 ff    POLL2 NORA #$FF
	 "00001111", "11111101",             -- 0018 - 0f fd          ADDA UARTCR
    "00100000", "11111101",             -- 001a - 20 fd          NORA #not(RXBF)
    "11101111", "11111000",             -- 001c - ef f8          BNE POLL2
	 "00100000", "11111111",             -- 001e - 20 ff          NORA #$FF
	 "00001111", "11111100",             -- 0020 - 0f fc          ADDA UARTDR
	 "01100111", "11011100",             -- 0022 - 67 dc          BRA RESET
	 "00000000", "00000000",             -- 0024 - 00 00          fcb $00,$00
	 "00000000", "00000000",             -- 0026 - 00 00          fcb $00,$00
	 "00000000", "00000000",             -- 0028 - 00 00          fcb $00,$00
	 "00000000", "00000000",             -- 002a - 00 00          fcb $00,$00
	 "00000000", "00000000",             -- 002c - 00 00          fcb $00,$00
	 "00000000", "00000000",             -- 002e - 00 00          fcb $00,$00
    "01001000", "01100101", "01101100", -- 0030 - 48 65 6c MSG   FCC "Hel"
	 "01101100", "01101111", "00100000", -- 0033 - 6c 6f 20       FCC "lo "
	 "01010111", "01101111", "01110010", -- 0036 - 57 6f 72       FCC "Wor"
    "01101100", "01100100",             -- 0039 - 6c 64          FCC "ld"
    "00001010", "00001101", "00000000", -- 003b - 0a 0d 00       FCB LF,CR,NULL
    "00000000", "00000000"              -- 003e - 00 00          fcb null,null           
);
begin
   WB_DAT_O <= rom_data(conv_integer(WB_ADR_I)); 
	WB_ACK_O <= WB_STB_I;
end architecture basic;
